@@ -19,14 +19,16 @@ arma::colvec FastcpdTest::GetGradientArma(const arma::mat& data,
   fastcpd::classes::Fastcpd fastcpd_instance(
       /* beta */ 0,
       /* cost */ R_NilValue,
+      /* cost_pelt */ nullptr,
+      /* cost_sen */ nullptr,
       /* cost_adjustment */ "MBIC",
-      /* cost_gradient */ R_NilValue,
-      /* cost_hessian */ R_NilValue,
+      /* cost_gradient */ nullptr,
+      /* cost_hessian */ nullptr,
       /* cp_only */ true,
       /* data */ data,
       /* epsilon */ 0.0,
       /* family */ "arma",
-      /* multiple_epochs_function */ R_NilValue,
+      /* multiple_epochs_function */ nullptr,
       /* line_search */ arma::colvec(),
       /* lower */ arma::colvec(),
       /* momentum_coef */ 0.0,
@@ -55,14 +57,16 @@ arma::mat FastcpdTest::GetHessianArma(const arma::mat& data,
   fastcpd::classes::Fastcpd fastcpd_instance(
       /* beta */ 0,
       /* cost */ R_NilValue,
+      /* cost_pelt */ nullptr,
+      /* cost_sen */ nullptr,
       /* cost_adjustment */ "MBIC",
-      /* cost_gradient */ R_NilValue,
-      /* cost_hessian */ R_NilValue,
+      /* cost_gradient */ nullptr,
+      /* cost_hessian */ nullptr,
       /* cp_only */ true,
       /* data */ data,
       /* epsilon */ 0.0,
       /* family */ "arma",
-      /* multiple_epochs_function */ R_NilValue,
+      /* multiple_epochs_function */ nullptr,
       /* line_search */ arma::colvec(),
       /* lower */ arma::colvec(),
       /* momentum_coef */ 0.0,
@@ -91,14 +95,16 @@ arma::mat FastcpdTest::GetHessianBinomial(const arma::mat& data,
   fastcpd::classes::Fastcpd fastcpd_instance(
       /* beta */ 0,
       /* cost */ R_NilValue,
+      /* cost_pelt */ nullptr,
+      /* cost_sen */ nullptr,
       /* cost_adjustment */ "MBIC",
-      /* cost_gradient */ R_NilValue,
-      /* cost_hessian */ R_NilValue,
+      /* cost_gradient */ nullptr,
+      /* cost_hessian */ nullptr,
       /* cp_only */ true,
       /* data */ data,
       /* epsilon */ 0.0,
       /* family */ "binomial",
-      /* multiple_epochs_function */ R_NilValue,
+      /* multiple_epochs_function */ nullptr,
       /* line_search */ arma::colvec(),
       /* lower */ arma::colvec(),
       /* momentum_coef */ 0.0,
@@ -127,14 +133,16 @@ arma::mat FastcpdTest::GetHessianPoisson(const arma::mat& data,
   fastcpd::classes::Fastcpd fastcpd_instance(
       /* beta */ 0,
       /* cost */ R_NilValue,
+      /* cost_pelt */ nullptr,
+      /* cost_sen */ nullptr,
       /* cost_adjustment */ "MBIC",
-      /* cost_gradient */ R_NilValue,
-      /* cost_hessian */ R_NilValue,
+      /* cost_gradient */ nullptr,
+      /* cost_hessian */ nullptr,
       /* cp_only */ true,
       /* data */ data,
       /* epsilon */ 0.0,
       /* family */ "poisson",
-      /* multiple_epochs_function */ R_NilValue,
+      /* multiple_epochs_function */ nullptr,
       /* line_search */ arma::colvec(),
       /* lower */ arma::colvec(),
       /* momentum_coef */ 0.0,
@@ -163,14 +171,16 @@ double FastcpdTest::GetNllSen(const arma::mat& data,
   fastcpd::classes::Fastcpd fastcpd_instance(
       /* beta */ 0,
       /* cost */ R_NilValue,
+      /* cost_pelt */ nullptr,
+      /* cost_sen */ nullptr,
       /* cost_adjustment */ "MBIC",
-      /* cost_gradient */ R_NilValue,
-      /* cost_hessian */ R_NilValue,
+      /* cost_gradient */ nullptr,
+      /* cost_hessian */ nullptr,
       /* cp_only */ true,
       /* data */ data,
       /* epsilon */ 0.0,
       /* family */ "arma",
-      /* multiple_epochs_function */ R_NilValue,
+      /* multiple_epochs_function */ nullptr,
       /* line_search */ arma::colvec(),
       /* lower */ arma::colvec(),
       /* momentum_coef */ 0.0,
@@ -193,21 +203,23 @@ double FastcpdTest::GetNllSen(const arma::mat& data,
 // GetNllPelt
 // Computes the negative log-likelihood for the PELT model.
 //------------------------------------------------------------------------------
-std::tuple<arma::mat, arma::mat, double> FastcpdTest::GetNllPelt(
+std::tuple<arma::colvec, arma::mat, double> FastcpdTest::GetNllPelt(
     const arma::mat& data, const unsigned int segment_start,
     const unsigned int segment_end, const bool cv,
     const Rcpp::Nullable<arma::colvec>& start) {
   fastcpd::classes::Fastcpd fastcpd_instance(
       /* beta */ 0,
       /* cost */ R_NilValue,
+      /* cost_pelt */ nullptr,
+      /* cost_sen */ nullptr,
       /* cost_adjustment */ "MBIC",
-      /* cost_gradient */ R_NilValue,
-      /* cost_hessian */ R_NilValue,
+      /* cost_gradient */ nullptr,
+      /* cost_hessian */ nullptr,
       /* cp_only */ true,
       /* data */ data,
       /* epsilon */ 0.0,
       /* family */ "arma",
-      /* multiple_epochs_function */ R_NilValue,
+      /* multiple_epochs_function */ nullptr,
       /* line_search */ arma::colvec(),
       /* lower */ arma::colvec(),
       /* momentum_coef */ 0.0,
