@@ -8,6 +8,6 @@ testthat::test_that(
       collapse = "\n"
     )))
 
-    testthat::expect_equal(result@cp_set, 209)
+    testthat::expect_equal(result@cp_set, c(203, 356))
   }
 )
